@@ -1,10 +1,11 @@
 import operator
+from builtins import all as all_
 
 from numpy.testing import assert_raises, suppress_warnings
 import numpy as np
 import pytest
 
-from .. import ones, asarray, reshape, result_type, all, equal
+from .. import ones, asarray, result_type, all, equal
 from .._array_object import Array, CPU_DEVICE
 from .._dtypes import (
     _all_dtypes,
@@ -21,8 +22,11 @@ from .._dtypes import (
     int32,
     int64,
     uint64,
+    float64,
     bool as bool_,
 )
+from .._flags import set_array_api_strict_flags
+
 import array_api_strict
 
 def test_validate_index():
@@ -315,7 +319,7 @@ def test_python_scalar_construtors():
 def test_device_property():
     a = ones((3, 4))
     assert a.device == CPU_DEVICE
-    assert a.device != 'cpu'
+    assert not isinstance(a.device, str)
 
     assert all(equal(a.to_device(CPU_DEVICE), a))
     assert_raises(ValueError, lambda: a.to_device('cpu'))
@@ -345,6 +349,17 @@ def test___array__():
     assert np.all(np.equal(b, np.ones((2, 3), dtype=np.float64)))
     assert b.dtype == np.float64
 
+def test_array_conversion():
+    # Check that arrays on the CPU device can be converted to NumPy
+    # but arrays on other devices can't
+    a = ones((2, 3))
+    np.asarray(a)
+
+    for device in ("device1", "device2"):
+        a = ones((2, 3), device=array_api_strict.Device(device))
+        with pytest.raises(RuntimeError, match="Can not convert array"):
+            np.asarray(a)
+
 def test_allow_newaxis():
     a = ones(5)
     indexed_a = a[None, :]
@@ -402,9 +417,57 @@ def test_array_keys_use_private_array():
 def test_array_namespace():
     a = ones((3, 3))
     assert a.__array_namespace__() == array_api_strict
+    assert array_api_strict.__array_api_version__ == "2023.12"
+
     assert a.__array_namespace__(api_version=None) is array_api_strict
+    assert array_api_strict.__array_api_version__ == "2023.12"
+
     assert a.__array_namespace__(api_version="2022.12") is array_api_strict
+    assert array_api_strict.__array_api_version__ == "2022.12"
+
+    assert a.__array_namespace__(api_version="2023.12") is array_api_strict
+    assert array_api_strict.__array_api_version__ == "2023.12"
+
     with pytest.warns(UserWarning):
         assert a.__array_namespace__(api_version="2021.12") is array_api_strict
+    assert array_api_strict.__array_api_version__ == "2021.12"
+
     pytest.raises(ValueError, lambda: a.__array_namespace__(api_version="2021.11"))
-    pytest.raises(ValueError, lambda: a.__array_namespace__(api_version="2023.12"))
+    pytest.raises(ValueError, lambda: a.__array_namespace__(api_version="2024.12"))
+
+def test_iter():
+    pytest.raises(TypeError, lambda: iter(asarray(3)))
+    assert list(ones(3)) == [asarray(1.), asarray(1.), asarray(1.)]
+    assert all_(isinstance(a, Array) for a in iter(ones(3)))
+    assert all_(a.shape == () for a in iter(ones(3)))
+    assert all_(a.dtype == float64 for a in iter(ones(3)))
+    pytest.raises(TypeError, lambda: iter(ones((3, 3))))
+
+@pytest.mark.parametrize("api_version", ['2021.12', '2022.12', '2023.12'])
+def dlpack_2023_12(api_version):
+    if api_version == '2021.12':
+        with pytest.warns(UserWarning):
+            set_array_api_strict_flags(api_version=api_version)
+    else:
+        set_array_api_strict_flags(api_version=api_version)
+
+    a = asarray([1, 2, 3], dtype=int8)
+    # Never an error
+    a.__dlpack__()
+
+
+    exception = NotImplementedError if api_version >= '2023.12' else ValueError
+    pytest.raises(exception, lambda:
+                  a.__dlpack__(dl_device=CPU_DEVICE))
+    pytest.raises(exception, lambda:
+                  a.__dlpack__(dl_device=None))
+    pytest.raises(exception, lambda:
+                  a.__dlpack__(max_version=(1, 0)))
+    pytest.raises(exception, lambda:
+                  a.__dlpack__(max_version=None))
+    pytest.raises(exception, lambda:
+                    a.__dlpack__(copy=False))
+    pytest.raises(exception, lambda:
+                    a.__dlpack__(copy=True))
+    pytest.raises(exception, lambda:
+                    a.__dlpack__(copy=None))
